%Classify data for NIMBLE using nearest neighbor density estimation
function Class_Prob = NN_KDE_Classify(x, x_location, net)

if isfield(net, 'location_weight')
    location_weight = net.location_weight;
else
    location_weight = 0.5;
end

norm_x = sqrt(sum(x .* x, 1));
norm_x(norm_x == 0) = 1;
x = bsxfun(@rdivide, x, norm_x);

%The max is used for numerical reasons with floating point arithmetic
feat_dist = max((2 - 2*(net.W * x)), 0);

%feat_dist = sqrt(feat_dist);

if ~isempty(x_location)  
    net_locations = net.W_Locations;
    location_dist = Compute_L2_Distance_Matrix(net_locations, x_location);
else
    location_dist = 0;
end

dist = feat_dist + location_weight*location_dist; 
Prob = 1 ./ (dist + 0.001);
Class_Prob = zeros(size(x, 2), net.num_classes);
for c = 1:net.num_classes
    mask = net.W_Class == c;        
    Class_Prob(:, c) = max(Prob(mask, :), [], 1);    
end
unknown = 0.001;

Class_Prob = bsxfun(@rdivide, Class_Prob,  sum(Class_Prob, 2) + eps('single'));
Class_Prob = Class_Prob + unknown;
Class_Prob = bsxfun(@rdivide, Class_Prob,  sum(Class_Prob, 2));
