%This function samples an image
function [patch_collection, location_collection, gist_feat, Correct_Time, Class_Prob, Model, Map_Data] = Process_Samples(operating_mode, img, class, num_samples, Model, Map_Data)

gist_feat = 0;
Map_Data = [];
if Model.reset_rand
    RandStream.setDefaultStream(RandStream('mt19937ar','seed',1));
end
if isfield(Model, 'PCA_Vectors') && ~isempty(Model.PCA_Vectors)
    reduce_dimensionality = true;
else
    reduce_dimensionality = false;
end

use_uniform_map = false;

Sequential_Test = false;

if operating_mode == 0
    random_policy = true;
else
    random_policy = false;
end

retina_radius = Model.retina_radius;

img_middle = [size(img, 1), size(img, 2)] ./ 2;

patch_collection = [];
location_collection = zeros(2, num_samples);
Class_Prob = [];


%Extract Features & saliency map
[Saliency_Map, Feature_Maps, ~, ~, Model.SUN] = Run_SUN(img, Model.SUN, inf, Model.img_size);

fm_norm = sqrt(sum(sum(Feature_Maps .* Feature_Maps, 1), 2));
Feature_Maps = bsxfun(@rdivide, Feature_Maps, fm_norm);


if use_uniform_map
    Saliency_Map = ones(size(Saliency_Map)) ./ numel(Saliency_Map);
end

Permutation_Mat = [];
Prior = [];
TD_Map = 1;
Correct_Time = zeros(1, num_samples, 'single');
entropy_hist = zeros(1, num_samples);


y_ndx = 0;
x_ndx = 0;

sorted_map = [];

%Now get fixations
for t = 1:num_samples
    if random_policy
        %look anywhere
        x_ndx = randn(1) * 4*sqrt(retina_radius) + img_middle(2);
        y_ndx = randn(1) * 4*sqrt(retina_radius) + img_middle(1);
    else
        %Look at regions based on BU saliency
        [new_center, Saliency_Map, sorted_map] = saccade(Saliency_Map, [y_ndx, x_ndx], retina_radius, sorted_map);
        x_ndx = new_center(2);
        y_ndx = new_center(1);
    end
    
    [patch, location, Permutation_Mat] = Get_NIMBLE_Features(Feature_Maps, y_ndx, x_ndx, retina_radius, Model.grid_list, Permutation_Mat);
    
    if reduce_dimensionality
        patch = Model.PCA_Vectors*bsxfun(@minus, patch, Model.PCA_Mean);
    end
    
    %store the patch if training
    if operating_mode == 1 || ~Sequential_Test %training
        if isempty(patch_collection)
            patch_collection = zeros(size(patch, 1), num_samples*size(patch, 2), 'single');
        end
        stop = t*size(patch, 2);
        start = 1+(stop-size(patch, 2));
        patch_collection(:, start:stop) = patch;
        location_collection(:, t) = location;
    end
    
    if operating_mode == 2 && Sequential_Test %sequential testing
        [Current_Prob, Class_Prob, scores] = Classify(Model, patch, location, Class_Prob, Prior);        
        [mc, e_c] = max(Class_Prob);
        Correct_Time(t) = class == e_c;        
        entropy_hist(t) = -sum(scores .* log(scores + realmin));
    end
end

if operating_mode == 2 && ~Sequential_Test %batch testing
    %this tends to be faster than sequential testing since it can be
    %vectorized in MATLAB. It may also be numerically more accurate.
    [Current_Prob, Class_Prob, scores] = Classify(Model, patch_collection, location_collection, Class_Prob, Prior);
    [m_val, m_ind] = max(Current_Prob, [], 2);
    Correct_Time = m_ind == class;
    Current_Prob = Current_Prob(end, :);
end


if operating_mode == 2
    Class_Prob = Current_Prob;
end